@font-face {
  font-family: 'icomoon';
  src:  url('fonts/icomoon.eot?vnck6d');
  src:  url('fonts/icomoon.eot?vnck6d#iefix') format('embedded-opentype'),
    url('fonts/icomoon.ttf?vnck6d') format('truetype'),
    url('fonts/icomoon.woff?vnck6d') format('woff'),
    url('fonts/icomoon.svg?vnck6d#icomoon') format('svg');
  font-weight: normal;
  font-style: normal;
}

[class^="icon-"], [class*=" icon-"] {
  /* use !important to prevent issues with browser extensions that change fonts */
  font-family: 'icomoon' !important;
  speak: none;
  font-style: normal;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;

  /* Better Font Rendering =========== */
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.icon-cross:before {
  content: "\e900";
}
.icon-diamond:before {
  content: "\e901";
}
.icon-square:before {
  content: "\e902";
}
.icon-star:before {
  content: "\e903";
}
.icon-triangle:before {
  content: "\e904";
}
.icon-uualone:before {
  content: "\e905";
}
.icon-uuwtext:before {
  content: "\e906";
}
.icon-wye:before {
  content: "\e907";
}
